@@ -37,6 +37,15 @@ def generator_is_parenthesised(asttokens: ASTTokens, node: ast.GeneratorExp) ->
     return False
 
 
+def bool_op_is_parenthesised(asttokens: ASTTokens, node: ast.BoolOp) -> bool:
+    prev_token = asttokens.prev_token(_first_token(node))
+    next_token = asttokens.next_token(_last_token(node))
+    if prev_token.string == '(' and next_token.string == ')':
+        return True
+
+    return False
+
+
 def node_start_position(asttokens: ASTTokens, node: ast.AST) -> Position:
     first_token = _first_token(node)
     if (
@@ -45,6 +54,12 @@ def node_start_position(asttokens: ASTTokens, node: ast.AST) -> Position:
     ):
         first_token = asttokens.prev_token(first_token)
 
+    elif (
+        isinstance(node, ast.BoolOp)
+        and bool_op_is_parenthesised(asttokens, node)
+    ):
+        first_token = asttokens.prev_token(first_token)
+
     return Position(*first_token.start)
 
 
@@ -82,9 +97,24 @@ def wrap_generator_body(
     return [(x, MutationType.WRAP_INDENT) for x in start_positions]
 
 
-def append_trailing_comma(summary: WrappingSummary, node: ast.AST) -> WrappingSummary:
+def append_trailing_comma(
+    asttokens: ASTTokens,
+    summary: WrappingSummary,
+    node: ast.AST,
+) -> WrappingSummary:
+    # Use the end position of the last content token, rather than the start
+    # position of the closing token. This ensures that we put the comma in the
+    # right place in constructs like:
+    #
+    #   func(
+    #       'abcd', 'defg'
+    #   )
+    #
+    # Where we want to put the comma immediately after 'defg' rather than just
+    # before the closing paren.
+    last_body_token = asttokens.prev_token(_last_token(node))
     summary.append((
-        Position.from_node_end(node),
+        Position(*last_body_token.end),
         MutationType.TRAILING_COMMA,
     ))
     return summary
@@ -155,7 +185,7 @@ def wrap_call(asttokens: ASTTokens, node: ast.Call) -> WrappingSummary:
         kwargs_stars = asttokens.prev_token(_first_token(kwargs))
         summary.append((Position(*kwargs_stars.start), MutationType.WRAP_INDENT))
 
-    append_trailing_comma(summary, node)
+    append_trailing_comma(asttokens, summary, node)
     append_wrap_end(summary, node)
     return summary
 
@@ -207,7 +237,7 @@ def wrap_dict(asttokens: ASTTokens, node: ast.Dict) -> WrappingSummary:
             positions.append(Position(*kwargs_stars.start))
 
     summary = [(x, MutationType.WRAP_INDENT) for x in positions]
-    append_trailing_comma(summary, node)
+    append_trailing_comma(asttokens, summary, node)
     append_wrap_end(summary, node)
     return summary
 
@@ -315,7 +345,7 @@ def wrap_if_exp(asttokens: ASTTokens, node: ast.IfExp) -> WrappingSummary:
 @node_wrapper(ast.List)
 def wrap_list(asttokens: ASTTokens, node: ast.List) -> WrappingSummary:
     summary = wrap_node_start_positions(asttokens, node.elts)
-    append_trailing_comma(summary, node)
+    append_trailing_comma(asttokens, summary, node)
     append_wrap_end(summary, node)
     return summary
 
@@ -329,8 +359,53 @@ def wrap_list_comp(asttokens: ASTTokens, node: ast.ListComp) -> WrappingSummary:
 
 @node_wrapper(ast.Tuple)
 def wrap_tuple(asttokens: ASTTokens, node: ast.Tuple) -> WrappingSummary:
+    first_token = _first_token(node)
+    last_token = _last_token(node)
+
+    is_parenthesised = first_token.string == '(' and last_token.string == ')'
+
+    def needs_parentheses_() -> bool:
+        prev_token = asttokens.prev_token(first_token)
+        next_token = asttokens.next_token(last_token)
+        if prev_token.string in '[' or next_token.string == ']':
+            return False
+
+        return True
+
+    needs_parentheses = needs_parentheses_()
+
     summary = wrap_node_start_positions(asttokens, node.elts)
-    if len(node.elts) > 1:
-        append_trailing_comma(summary, node)
-    append_wrap_end(summary, node)
+
+    if not is_parenthesised:
+        if needs_parentheses:
+            summary.insert(0, (
+                Position.from_node_start(node),
+                MutationType.OPEN_PAREN,
+            ))
+
+        end_pos = Position(*_last_token(node).end)
+
+        if len(node.elts) > 1:
+            summary.append((
+                end_pos,
+                MutationType.TRAILING_COMMA,
+            ))
+
+        summary.append((
+            end_pos,
+            MutationType.WRAP,
+        ))
+
+        if needs_parentheses:
+            summary.append((
+                end_pos,
+                MutationType.CLOSE_PAREN,
+            ))
+
+    else:
+        if len(node.elts) > 1:
+            append_trailing_comma(asttokens, summary, node)
+
+        append_wrap_end(summary, node)
+
     return summary
